"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinkedList = void 0;
class LinkedList {
    constructor(arr) {
        this.size = 0;
        if (arr && arr[Symbol.iterator]) {
            for (const item of arr) {
                this._push(item);
            }
        }
    }
    push(...items) {
        for (const item of items) {
            this._push(item);
        }
    }
    get head() {
        var _a;
        return (_a = this._head) === null || _a === void 0 ? void 0 : _a.value;
    }
    get tail() {
        var _a;
        return (_a = this._tail) === null || _a === void 0 ? void 0 : _a.value;
    }
    _push(item) {
        this.size += 1;
        if (!this._tail) {
            this._head = this._tail = {
                value: item,
            };
            return;
        }
        const newItem = {
            value: item,
            prev: this._tail,
        };
        this._tail.next = newItem;
        this._tail = newItem;
    }
    pushFront(...items) {
        for (const item of items) {
            this._pushFront(item);
        }
    }
    _pushFront(item) {
        this.size += 1;
        if (!this._head) {
            this._head = this._tail = {
                value: item,
            };
            return;
        }
        const newItem = {
            value: item,
            next: this._head,
        };
        this._head.prev = newItem;
        this._head = newItem;
    }
    pop() {
        if (!this._tail) {
            return;
        }
        this.size -= 1;
        const last = this._tail;
        this._tail = last.prev;
        this._updateHeadAndTail();
        return last.value;
    }
    popFront() {
        if (!this._head) {
            return;
        }
        this.size -= 1;
        const first = this._head;
        this._head = first.next;
        this._updateHeadAndTail();
        return first.value;
    }
    _updateHeadAndTail() {
        if (!this._head || !this._tail || this.size === 0) {
            this._head = this._tail = undefined;
            this.size = 0;
            return;
        }
        this._head.prev = undefined;
        this._tail.next = undefined;
    }
    get empty() {
        return this.size === 0;
    }
    clear() {
        this._head = undefined;
        this._tail = undefined;
        this.size = 0;
    }
    *[Symbol.iterator]() {
        if (!this._head) {
            return;
        }
        let node = this._head;
        while (node) {
            yield node.value;
            node = node.next;
        }
    }
    toArray() {
        return Array.from(this);
    }
}
exports.LinkedList = LinkedList;
