"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AsyncQueue = void 0;
const Queue_1 = require("../data-structure/Queue");
const utils_1 = require("../utils");
const Lock_1 = require("./Lock");
class AsyncQueue {
    constructor() {
        this.queue = new Queue_1.Queue();
        this.lock = new Lock_1.Lock();
        this.setPushRelease();
    }
    setPushRelease() {
        this.lock.acquireImmediately().then((r) => {
            this.pushRelease = r;
        });
    }
    async pop(timeout) {
        if (this.queue.size) {
            return this.queue.pop();
        }
        const popRelease = await this.lock.acquire(timeout);
        try {
            return this.queue.pop();
        }
        finally {
            if (this.queue.size === 0) {
                this.pushRelease = popRelease;
            }
            else {
                popRelease();
            }
        }
    }
    push(v) {
        this._push(v);
    }
    async _push(v) {
        this.queue.push(v);
        if (!this.pushRelease) {
            await utils_1.delay(0);
        }
        this.pushRelease && this.pushRelease();
    }
    get size() {
        return this.queue.size;
    }
    clear() {
        this.queue.clear();
        this.lock = new Lock_1.Lock();
        this.pushRelease = undefined;
        this.setPushRelease();
    }
}
exports.AsyncQueue = AsyncQueue;
