"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.progressiveGenerate = void 0;
const delay_1 = require("../../utils/delay");
const requestIdleCallback_1 = require("../../utils/requestIdleCallback");
function progressiveGenerate(generator, { maxExecutionDuration = 2, minInterval = 0, ignoreOutput = false, useIdleCallback = false, } = {}) {
    let done = false;
    const run = async () => {
        const ans = [];
        let startTime = Date.now();
        for (const value of generator) {
            !ignoreOutput && ans.push(value);
            if (Date.now() >= startTime + maxExecutionDuration) {
                await delay_1.delay(minInterval);
                if (useIdleCallback) {
                    await requestIdleCallback_1.awaitIdle();
                }
                startTime = Date.now();
            }
            if (done) {
                return ans;
            }
        }
        return ans;
    };
    return {
        promise: run(),
        abort: () => {
            generator.return(undefined);
            done = true;
        },
    };
}
exports.progressiveGenerate = progressiveGenerate;
