"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskQueue = void 0;
const Queue_1 = require("../../data-structure/Queue");
const ErrorHandler_1 = require("./ErrorHandler");
const TaskWorker_1 = require("./TaskWorker");
class TaskQueue {
    constructor({ maxParallelNum = 1, taskInterval = 0, onError = ErrorHandler_1.DefaultErrorHandler, useIdleWorker = false, } = {}) {
        this.workers = [];
        this.maxParallelNum = 1;
        this.taskInterval = 0;
        this.queue = new Queue_1.Queue();
        this.maxParallelNum = Math.max(maxParallelNum, 1);
        this.taskInterval = taskInterval;
        this.onError = onError;
        this.useIdleWorker = useIdleWorker;
    }
    push(task) {
        let resolve;
        let reject;
        const promise = new Promise((_resolve, _reject) => {
            resolve = _resolve;
            reject = _reject;
        });
        if (task instanceof Function) {
            this.queue.push({ callback: task, reject, resolve });
        }
        else {
            this.queue.push({ ...task, resolve, reject });
        }
        this.start();
        return promise;
    }
    start() {
        let targetParallelNum = Math.min(this.queue.size + this.runningNum, this.maxParallelNum);
        while (this.workers.length < targetParallelNum) {
            const worker = this.useIdleWorker
                ? new TaskWorker_1.IdleWorker(this.queue, this.onError)
                : new TaskWorker_1.TaskWorker(this.queue, this.onError, this.taskInterval);
            this.workers.push(worker);
        }
        for (const worker of this.workers) {
            if (targetParallelNum === 0) {
                break;
            }
            targetParallelNum -= 1;
            worker.run();
        }
    }
    get runningNum() {
        return this.workers.reduce((l, r) => l + (r.busy ? 1 : 0), 0);
    }
    pause() {
        this.workers.forEach((x) => x.destroy());
        this.workers = [];
    }
    destroy() {
        this.pause();
        this.queue.clear();
    }
}
exports.TaskQueue = TaskQueue;
