"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IdleWorker = void 0;
const requestIdleCallback_1 = require("../../../utils/requestIdleCallback");
const BaseTaskWorker_1 = require("./BaseTaskWorker");
class IdleWorker extends BaseTaskWorker_1.BaseTaskWorker {
    constructor() {
        super(...arguments);
        this.remaining = () => 0;
        this.estimatedExecutionTime = 1;
    }
    preExecute() {
        if (this.remaining() >= this.estimatedExecutionTime) {
            return;
        }
        return requestIdleCallback_1.awaitIdle().then(({ timeRemaining }) => {
            this.remaining = timeRemaining;
        });
    }
    async runTask(task) {
        const startTime = Date.now();
        await super.runTask(task);
        const duration = Date.now() - startTime;
        if (duration > this.estimatedExecutionTime) {
            this.estimatedExecutionTime = duration;
        }
    }
}
exports.IdleWorker = IdleWorker;
