import { Queue } from "../data-structure/Queue";
import { delay } from "../utils";
import { Lock } from "./Lock";
export class AsyncQueue {
    constructor() {
        this.queue = new Queue();
        this.lock = new Lock();
        this.setPushRelease();
    }
    setPushRelease() {
        this.lock.acquireImmediately().then((r) => {
            this.pushRelease = r;
        });
    }
    async pop(timeout) {
        if (this.queue.size) {
            return this.queue.pop();
        }
        const popRelease = await this.lock.acquire(timeout);
        try {
            return this.queue.pop();
        }
        finally {
            if (this.queue.size === 0) {
                this.pushRelease = popRelease;
            }
            else {
                popRelease();
            }
        }
    }
    push(v) {
        this._push(v);
    }
    async _push(v) {
        this.queue.push(v);
        if (!this.pushRelease) {
            await delay(0);
        }
        this.pushRelease && this.pushRelease();
    }
    get size() {
        return this.queue.size;
    }
    clear() {
        this.queue.clear();
        this.lock = new Lock();
        this.pushRelease = undefined;
        this.setPushRelease();
    }
}
