import { LinkedList } from "../data-structure";
export class LinkedPromise {
    constructor() {
        this.list = new LinkedList();
        this.running = false;
        this.paused = false;
    }
    pushTask() {
        const promiseItem = createPromiseItem();
        this.list.push(promiseItem);
        this.start();
        return promiseItem.promise;
    }
    pushTaskFront() {
        const promiseItem = createPromiseItem();
        this.list.pushFront(promiseItem);
        this.start();
        return promiseItem.promise;
    }
    pause() {
        this.paused = true;
    }
    start() {
        this._run();
    }
    async _run() {
        this.paused = false;
        if (this.running) {
            return;
        }
        this.running = true;
        while (this.list.size) {
            if (this.paused) {
                this.running = false;
                return;
            }
            const front = this.list.popFront();
            await new Promise((r) => front.resolve(r));
        }
    }
    destroy() {
        this.list.clear();
        this.paused = true;
    }
}
function createPromiseItem() {
    let resolve;
    let reject;
    return {
        promise: new Promise((_resolve, _reject) => {
            reject = _reject;
            resolve = _resolve;
        }),
        resolve: resolve,
        reject: reject,
    };
}
