import { delay } from "../../utils/delay";
import { awaitIdle } from "../../utils/requestIdleCallback";
export function progressiveGenerate(generator, { maxExecutionDuration = 2, minInterval = 0, ignoreOutput = false, useIdleCallback = false, } = {}) {
    let done = false;
    const run = async () => {
        const ans = [];
        let startTime = Date.now();
        for (const value of generator) {
            !ignoreOutput && ans.push(value);
            if (Date.now() >= startTime + maxExecutionDuration) {
                await delay(minInterval);
                if (useIdleCallback) {
                    await awaitIdle();
                }
                startTime = Date.now();
            }
            if (done) {
                return ans;
            }
        }
        return ans;
    };
    return {
        promise: run(),
        abort: () => {
            generator.return(undefined);
            done = true;
        },
    };
}
