import { TaskQueue } from "./TaskQueue";
export class IntervalTask {
    constructor(task, interval) {
        this.task = task;
        this._stopped = true;
        this.loop = (value) => {
            if (this._stopped) {
                return;
            }
            this._latestValue = value;
            this.queue.push(this.task).then(this.loop);
        };
        this.queue = new TaskQueue({ maxParallelNum: 1, taskInterval: interval });
    }
    start() {
        this._stopped = false;
        this.loop(this._latestValue);
    }
    stop() {
        this.queue.pause();
        this._stopped = true;
    }
    get stopped() {
        return this._stopped;
    }
    get running() {
        return !this._stopped;
    }
    get latestValue() {
        return this._latestValue;
    }
    destroy() {
        this.stop();
        this.queue.destroy();
        this._latestValue = undefined;
    }
}
