export class BaseTaskWorker {
    constructor(queue, onError) {
        this.queue = queue;
        this.onError = onError;
        this._busy = false;
        this._destroyed = false;
    }
    preExecute() {
        return;
    }
    postExecute() {
        return;
    }
    async run() {
        if (this._busy) {
            return;
        }
        this._busy = true;
        try {
            while (this.queue.size && !this._destroyed) {
                const task = this.queue.pop();
                /* istanbul ignore if */
                if (!task) {
                    continue;
                }
                const promise = this.preExecute();
                promise && (await promise);
                await this.runTask(task);
                const promise_ = this.postExecute();
                promise_ && (await promise_);
            }
        }
        finally {
            this._busy = false;
        }
    }
    async runTask(task) {
        try {
            const res = await task.callback();
            task.resolve(res);
        }
        catch (e) {
            this.onError(e);
            task.reject(e);
        }
    }
    get busy() {
        return this._busy;
    }
    destroy() {
        this._destroyed = true;
    }
}
